@@ -1,5 +1,10 @@
+import contextlib
 import io
 import os
+import random
+import re
+import shlex
+import urllib.request
 from contextlib import contextmanager
 from typing import BinaryIO, Dict
 
@@ -8,8 +13,18 @@ import rich.console as rich_console
 import validators
 
 import fixieai.client
+from fixieai import constants
 from fixieai.cli.agent import agent_config
 from fixieai.cli.agent import loader
+from fixieai.cli.agent import tunnel as tunnel_
+
+# Regex pattern to match valid entry points: "module:object"
+VAR_NAME_RE = r"(?![0-9])\w+"
+ENTRY_POINT_PATTERN = re.compile(rf"^{VAR_NAME_RE}(\.{VAR_NAME_RE})*:{VAR_NAME_RE}$")
+# The agent template main.py file
+AGENT_TEMPLATE_URL = (
+    "https://raw.githubusercontent.com/fixie-ai/fixie-examples/main/agents/template.py"
+)
 
 
 @click.group(help="Agent-related commands.")
@@ -31,7 +46,16 @@ def _validate_slug(ctx, param, value):
 def _validate_url(ctx, param, value):
     while True:
         if value and not validators.url(value):
-            click.secho(f"{param.name} must be a valid url.")
+            click.echo(f"{param.name} must be a valid url.")
+            value = click.prompt(param.prompt, default=param.default())
+        else:
+            return value
+
+
+def _validate_entry_point(ctx, param, value):
+    while True:
+        if value and not ENTRY_POINT_PATTERN.match(value):
+            click.echo(f"{param.name} must be in module:obj format (e.g. 'main:agent')")
             value = click.prompt(param.prompt, default=param.default())
         else:
             return value
@@ -51,8 +75,9 @@ def _validate_url(ctx, param, value):
 )
 @click.option(
     "--entry-point",
-    prompt=True,
+    prompt="Entry point (module:object)",
     default=lambda: _current_config().entry_point,
+    callback=_validate_entry_point,
 )
 @click.option(
     "--more-info-url",
@@ -78,6 +103,17 @@ def init_agent(handle, description, entry_point, more_info_url, public):
     current_config.public = public
     agent_config.save_config(current_config)
 
+    entry_module, _ = entry_point.split(":")
+    expected_main_path = entry_module.replace(".", "/") + ".py"
+    if not os.path.exists(expected_main_path):
+        urllib.request.urlretrieve(AGENT_TEMPLATE_URL, expected_main_path)
+        click.secho(
+            f"Initialized agent.yaml and made a template agent file at {expected_main_path}",
+            fg="green",
+        )
+    else:
+        click.secho(f"Initialized agent.yaml.", fg="green")
+
 
 def _current_config() -> agent_config.AgentConfig:
     """Loads current agent config, or a default if not initialized."""
@@ -93,7 +129,9 @@ def _current_config() -> agent_config.AgentConfig:
 def list_agents(ctx, verbose):
     client = ctx.obj.client
     agents = client.get_agents()
-    for agent_id, agent in agents.items():
+    agent_ids = sorted(agents.keys())
+    for agent_id in agent_ids:
+        agent = agents[agent_id]
         click.secho(f"{agent_id}", fg="green", nl=False)
         click.echo(f": {agent['name']}")
         if verbose:
@@ -104,6 +142,44 @@ def list_agents(ctx, verbose):
             click.echo()
 
 
+@agent.command("show", help="Show an agent.")
+@click.argument("agent_id")
+@click.pass_context
+def show_agent(ctx, agent_id: str):
+    client = ctx.obj.client
+    agent = client.get_agent(agent_id)
+    if not agent.valid:
+        click.echo(f"Agent {agent_id} not found.")
+        return
+    click.secho(f"{agent.agent_id}", fg="green", nl=False)
+    click.echo(f"Owner: {agent.owner}")
+    if agent.name:
+        click.echo(f": {agent.name}")
+    if agent.description:
+        click.echo(f"Description: {agent.description}")
+    if agent.more_info_url:
+        click.echo(f"More info URL: {agent.more_info_url}")
+    click.echo(f"Published: {agent.published}")
+    if agent.func_url:
+        click.echo(f"Func URL: {agent.func_url}")
+    if agent.query_url:
+        click.echo(f"Query URL: {agent.query_url}")
+    click.echo(f"Created: {agent.created}")
+    click.echo(f"Modified: {agent.modified}")
+    if agent.queries:
+        click.echo(f"Queries: {agent.queries}")
+
+
+@agent.command("delete", help="Delete an agent.")
+@click.argument("handle")
+@click.pass_context
+def delete_agent(ctx, handle: str):
+    client = ctx.obj.client
+    agent = client.get_agent(handle)
+    agent.delete_agent()
+    click.echo(f"Deleted agent {agent.agent_id}")
+
+
 def _validate_agent_path(ctx, param, value):
     normalized = agent_config.normalize_path(value)
     if not os.path.exists(normalized):
@@ -115,8 +191,8 @@ def _ensure_agent_updated(
     client: fixieai.client.FixieClient,
     config: agent_config.AgentConfig,
 ):
-    agent = client.get_agent(config.handle)
-    if agent.agent_id is None:
+    agent = client.get_agent(f"{client.get_current_username()}/{config.handle}")
+    if not agent.valid:
         agent.create_agent(
             name=config.name,
             description=config.description,
@@ -137,16 +213,35 @@ def _ensure_agent_updated(
     return agent
 
 
-@agent.command("serve", help="Start serving the current agent.")
+@agent.command(
+    "serve", help="Serve the current agent locally via a publicly-accessible URL."
+)
 @click.argument("path", callback=_validate_agent_path, required=False)
 @click.option("--host", default="0.0.0.0")
 @click.option("--port", type=int, default=8181)
+@click.option(
+    "--tunnel/--no-tunnel",
+    is_flag=True,
+    default=True,
+    help="Create a tunnel using localhost.run",
+)
 @click.pass_context
-def serve(ctx, path, host, port):
-    # TODO: add tunneling and deployment url update
+def serve(ctx, path, host, port, tunnel):
+    console = rich_console.Console()
     config, agent_impl = loader.load_agent_from_path(path)
-    agent_api = _ensure_agent_updated(ctx.obj.client, config)
-    agent_impl.serve(agent_api.agent_id, host, port)
+
+    with contextlib.ExitStack() as stack:
+        if tunnel:
+            # Start up a tunnel via localhost.run.
+            console.print(f"Opening tunnel to {host}:{port} via localhost.run.")
+            console.print(
+                f"[yellow]This replaces any existing deployment, run [bold]fixie deploy[/bold] to redeploy to prod.[/yellow]"
+            )
+            config.deployment_url = stack.enter_context(tunnel_.Tunnel(host, port))
+            console.print(f"{host}:{port} can be reached at {config.deployment_url}")
+
+        agent_api = _ensure_agent_updated(ctx.obj.client, config)
+        agent_impl.serve(agent_api.agent_id, host, port)
 
 
 _DEPLOYMENT_BOOTSTRAP_SOURCE = """
@@ -174,13 +269,18 @@ def _spinner(console: rich_console.Console, text: str):
 
 @agent.command("deploy", help="Deploy the current agent.")
 @click.argument("path", callback=_validate_agent_path, required=False)
+@click.option(
+    "--metadata-only",
+    is_flag=True,
+    help="Only publish metadata and refresh, do not redeploy.",
+)
 @click.pass_context
-def deploy(ctx, path):
+def deploy(ctx, path, metadata_only):
     console = rich_console.Console()
     config = agent_config.load_config(path)
     agent_api = _ensure_agent_updated(ctx.obj.client, config)
 
-    if config.deployment_url is None:
+    if config.deployment_url is None and not metadata_only:
         # Deploy the agent to fixie with some bootstrapping code.
         file_streams: Dict[str, BinaryIO] = {}
         deploy_root = os.path.dirname(path)
@@ -222,3 +322,14 @@ def deploy(ctx, path):
     # Trigger a refresh with the updated deployment
     with _spinner(console, "Refreshing..."):
         ctx.obj.client.refresh_agent(config.handle)
+
+    agent_api.update_agent()
+    if agent_api.queries:
+        suggested_query = random.choice(agent_api.queries)
+    else:
+        suggested_query = "Hello!"
+
+    suggested_message = shlex.quote(f"@{agent_api.agent_id} {suggested_query}")
+    console.print(
+        f"Your agent was deployed to {constants.FIXIE_API_URL}/agents/{agent_api.agent_id}\nYou can also chat with your agent using the fixie CLI:\n\nfixie console {suggested_message}"
+    )
